
! Inverse Distance Square interpolation
! Avoid masked points (e.g. Land ) during interpolation
! Author: Biju Thomas  on 2016/01/12 

MODULE invdistinpmd
IMPLICIT NONE
  INTEGER,  PARAMETER :: flagint = -999999
  INTEGER :: istep = 0
  REAL, PARAMETER :: flagreal = 1.0e29, noinpval = 0.0
  REAL, PARAMETER :: pi = 3.141592653589
  REAL, PARAMETER :: d2rad = pi/180.0
  REAL, PARAMETER :: pow = 2.0
  REAL, PARAMETER :: eps = 8.0e-02

  CONTAINS

  SUBROUTINE invdistinp(maskin,maskout,fin,fout,   &
                       i1, j1, i2, j2,w11, w12, w21, w22, nx,ny)

! Doing  interpolation 2dim fields
! Author: Biju Thomas  on 2015/01/12 ....

  IMPLICIT NONE
    REAL, INTENT(IN) :: maskin,maskout
    INTEGER, INTENT(IN) :: nx, ny, i1, j1, i2, j2
    REAL, INTENT(IN) :: w11, w12, w21, w22
    REAL, INTENT(IN), DIMENSION(nx,ny) :: fin
    REAL, INTENT(OUT) :: fout
    REAL :: wt11, wt12, wt21, wt22
    INTEGER :: i, j
    REAL :: sum
      
    IF ( w11 == noinpval ) THEN
      fout = fin(i1,j1)
      RETURN
    ELSEIF ( w12 == noinpval ) THEN
      fout = fin(i1,j2)
      RETURN
    ELSEIF ( w21 == noinpval ) THEN
      fout = fin(i2,j1)
      RETURN
    ELSEIF ( w22 == noinpval ) THEN
      fout = fin(i2,j2)
      RETURN
    ENDIF

    IF ( w11 == flagreal ) THEN
      wt11 = 0.0
    ELSE
      wt11 = w11
    ENDIF
    IF ( w12 == flagreal ) THEN
      wt12 = 0.0
    ELSE
      wt12 = w12
    ENDIF
    IF ( w21 == flagreal ) THEN
      wt21 = 0.0
    ELSE
      wt21 = w21
    ENDIF
    IF ( w22 == flagreal ) THEN
       wt22 = 0.0
    ELSE
       wt22 = w22 
    ENDIF

    sum = wt11 + wt12 + wt21 + wt22
    IF ( sum /= 0.0 ) THEN
       fout = ( wt11*fin(i1,j1) + wt12*fin(i1,j2) +                    &
                wt21*fin(i2,j1) + wt22*fin(i2,j2) ) / sum
    ELSE
       fout = maskout
    ENDIF

    
    RETURN
    END SUBROUTINE invdistinp

 
  FUNCTION monotonic (nn, g1d) RESULT(check)
! Check grid points are monotonically increaing
! Author: Biju Thomas  on 2016/01/12 ....

  IMPLICIT NONE
    INTEGER, INTENT(IN) :: nn
    REAL, DIMENSION(nn), INTENT(IN) :: g1d
    LOGICAL :: check
    INTEGER :: k

    check = .TRUE.
    DO k = 1, nn-1
      IF ( g1d(k) > g1d(k+1) ) THEN
        check = .FALSE.
        RETURN
      ENDIF
    ENDDO 
  END FUNCTION

  SUBROUTINE nearpts(nx, ny, g2dx, g2dy, xi, yi, is, ie,            &
                     js, je, ilo, jlo, iup, jup )
! Doing  searching  4  neighboring points.....
! Author: Biju Thomas  on 2016/01/12 ....
! g2dx, g2dy need to monotonically increaing

  IMPLICIT NONE
    INTEGER, INTENT(IN) :: nx, ny, is, ie, js, je
    INTEGER, INTENT(OUT) :: ilo, jlo, iup, jup
    REAL, INTENT(IN) :: xi, yi
    REAL, DIMENSION(nx, ny), INTENT(IN) :: g2dx, g2dy
    INTEGER :: i, j
    LOGICAL :: loopexit
    
    IF ( istep == 0 ) THEN
      DO j = 1, ny
        IF ( .NOT. monotonic (nx, g2dx(:,j)) )  THEN
          PRINT*, 'INVALID INPUT GRID(lono: invdistinp) EXIT', j
          RETURN
        ENDIF
      ENDDO
      DO i = 1, nx
        IF ( .NOT. monotonic (ny, g2dy(i,:)) )  THEN
          PRINT*, 'INVALID INPUT GRID(lato: invdistinp) EXIT', i
          RETURN
        ENDIF
      ENDDO
    ENDIF
    istep =  1
    IF ( is > ie .OR. js > je) THEN
      PRINT*, 'INVALID is, ie, js, je (invdistinp) EXIT', is, ie, js, je
      RETURN
    ENDIF
    loopexit = .FALSE.
    ilo = flagint 
    jlo = flagint
    iup = flagint
    jup = flagint
    DO j = js, je-1
    DO i = is, ie-1
      IF ( xi >= g2dx(i,j) .AND. xi <= g2dx(i+1,j) .AND.       &
           yi >= g2dy(i,j) .AND. yi <= g2dy(i,j+1) ) THEN
        ilo = i
        jlo = j
        iup = i+1
        jup = j+1
        loopexit = .TRUE.
        EXIT
      ENDIF
    ENDDO
    IF ( loopexit ) EXIT
    ENDDO
   
    IF ( ilo == flagint .OR. jlo == flagint .OR. iup == flagint .OR. &
         jup == flagint ) THEN
      DO j = js, je
      DO i = is, ie
        IF ( xi >= g2dx(i,j)-eps .AND. xi <= g2dx(i+1,j)+eps .AND.       &
             yi >= g2dy(i,j)-eps .AND. yi <= g2dy(i,j+1)+eps ) THEN
          ilo = i
          jlo = j
          iup = i+1
          jup = j+1
          loopexit = .TRUE.
          EXIT
        ENDIF
      ENDDO
      IF ( loopexit ) EXIT
      ENDDO
    ENDIF
  END SUBROUTINE  nearpts
  
  SUBROUTINE weights(nx, ny, g2dx, g2dy, xi, yi, fin,     &
            ilo, jlo, iup, jup, w11, w12, w21, w22, maskin)
! Calculating weights.....
! Author: Biju Thomas  on 2016/01/12 ....
! g2dx, g2dy need to monotonically increaing

  IMPLICIT NONE 
    INTEGER, INTENT(IN) :: nx, ny
    INTEGER, INTENT(IN) :: ilo, jlo, iup, jup
    REAL, INTENT(IN) :: xi, yi, maskin 
    REAL, INTENT(OUT) :: w11, w12, w21, w22
    REAL, DIMENSION(nx, ny), INTENT(IN) :: fin, g2dx, g2dy
    INTEGER :: i, j

    IF ( ilo == flagint .OR. jlo == flagint .OR. iup == flagint .OR. jup == flagint ) THEN
     PRINT*,'FAILED TO FIND NEAR POINTS', ilo, iup, jlo, jup, xi, yi
     PRINT*,'EXIT(invdistinp.f90)'
     RETURN
    ENDIF
    IF ( ilo == flagint .OR. jlo == flagint .OR. fin(ilo,jlo) == maskin ) THEN
       w11 =  flagreal
    ELSE
       w11 = invdiswt(g2dx(ilo,jlo), g2dy(ilo,jlo), xi, yi, pow)
    ENDIF
    IF ( ilo == flagint .OR. jup == flagint .OR. fin(ilo,jup) == maskin ) THEN
       w12 =  flagreal
    ELSE
       w12 = invdiswt(g2dx(ilo,jup), g2dy(ilo,jup), xi, yi, pow)
    ENDIF
    IF ( iup == flagint .OR. jlo == flagint .OR. fin(iup,jlo) == maskin ) THEN
       w21 =  flagreal
    ELSE
       w21 = invdiswt(g2dx(iup,jlo), g2dy(iup,jlo), xi, yi, pow)
    ENDIF
    IF ( iup == flagint .OR. jup == flagint .OR. fin(iup,jup) == maskin ) THEN
       w22 =  flagreal
    ELSE
       w22 = invdiswt(g2dx(iup,jup), g2dy(iup,jup), xi, yi, pow)
    ENDIF
  END SUBROUTINE weights

  FUNCTION gcirdist(lon1, lat1, lon2, lat2) RESULT(dist)
  IMPLICIT NONE
     REAL, INTENT(IN) :: lon1, lat1, lon2, lat2
     REAL :: rlat1, rlat2
     REAL :: dlonr, dist
     IF ( lon1 == lon2 .AND. lat1 == lat2 ) THEN
        dist = 0.0
        RETURN
     ENDIF    
     dlonr = ABS(lon1 - lon2) * d2rad
     rlat1 = lat1 * d2rad
     rlat2 = lat2 * d2rad
     dist = ACOS(SIN(rlat1)*SIN(rlat2)+                &
               COS(rlat1)*COS(rlat2)*COS(dlonr))/d2rad
  END FUNCTION 

  FUNCTION invdiswt(lon1, lat1, lon2, lat2, pow) RESULT(wt)
  IMPLICIT NONE
     REAL, INTENT(IN) :: lon1, lat1, lon2, lat2, pow
     REAL :: dist, wt
     dist = gcirdist(lon1, lat1, lon2, lat2)
     IF (dist == 0.0) THEN 
        wt = noinpval
     ELSE
        wt = 1.0 / (dist**pow)
     ENDIF 
  END FUNCTION

  SUBROUTINE check_weights(maskin, f11, f12, f21, f22, w11, w12, w21, w22 )
  IMPLICIT NONE    
     REAL, INTENT(IN) :: maskin
     REAL, INTENT(INOUT) :: w11, w12, w21, w22
     REAL, INTENT(IN) :: f11, f12, f21, f22
       IF (f11 == maskin ) THEN
          w11 =  flagreal
       ENDIF
       IF (f12 == maskin ) THEN
          w12 =  flagreal
       ENDIF
       IF (f21 == maskin ) THEN
          w21 =  flagreal
       ENDIF
       IF (f22 == maskin ) THEN
          w22 =  flagreal
       ENDIF
  END SUBROUTINE check_weights

END MODULE invdistinpmd
